/*-------------------------------------------*\
 |  EJEMPLO 3.10                             |
 |  Autor: Pedro M Jimnez                  |
 |  (c) Microsystems Engineering (Bilbao)    |
\*-------------------------------------------*/

// Ejemplo para MicroPIC Trainer y Trainer PLUS

// Comunicacin serie gestionada por software.

// Haciendo uso de las rutinas incluidas las librerias del compilador C, se trata
// de utilizar el canal RS232 de la Trainer PLUS para comunicar va serie con 
// diferentes perifricos. En los ejemplos aqu propuestos se utilizar un PC para
// que, con el software TERMINAL.EXE, actue como un simple terminal de comunicaciones.
// El usuario deber preparar los cables de comunicacin entre la Trainer PLUS y el
// PC, teniendo en cuenta de cruzar la seal de transmisin (TxD) con la de
// recepcin (RxD) y viceversa.

// Otro ejemplo de comunicaciones. Imaginemos que, desde un terminal, se desea
// comandar el movimiento de una estructura mvil, segn la siguiente tabla:
//
//       TECLA  MOVIMIENTO
//       -----  ----------
//         Q    Adelante
//         A    Atras
//         O    Izquierda
//         P    Derecha
//        SPC   Parada de emergencia
//
// La pulsacin de cualquiera de ellas hace bascular el estado de las salidas
// RB3-RB0 que controlan los elementos motrices de la siguiente forma:
//
//
//       SALIDA MOVIENTO
//       ------ --------
//         RB0  Adelante
//         RB1  Atras
//         RB2  Izquierda
//         RB3  Derecha
//
// Finalmente, el PIC devuelve al terminal el tipo de movimiento que est realizando.


#include <16f84.h>

#use delay(clock=4000000,RESTART_WDT)
#use RS232(BAUD=9600, BITS=8, PARITY=N, XMIT=PIN_B4, RCV=PIN_B5, RESTART_WDT)
#use fast_io(B)

#byte port_b = 6


main()
{
  short Adelante,Atras,Izquierda,Derecha;

  port_b = 0b00010000;
  set_tris_b(0x20);

  Adelante=Atras=Izquierda=Derecha=FALSE;

  while(1)
  {
    restart_wdt();

    output_bit(PIN_B0,Adelante);
    output_bit(PIN_B1,Atras);
    output_bit(PIN_B2,Izquierda);
    output_bit(PIN_B3,Derecha);

    if(!(port_b & 0x0F))
      puts("Situacion NULA");

    if(Adelante && !Izquierda && !Derecha)
      puts("Marcha adelante.");
    if(Atras && !Izquierda && !Derecha)    
      puts("Marcha atras.")
    if(Izquierda && !Adelante && !Atras)
      puts("Mueve a Izquierdas.");
    if(Derecha && !Adelante && !Atras)
      puts("Mueve a Derechas.");

    if(Adelante && Izquierda)
      puts("Adelante/Izquierda.");
    if(Adelante && Derecha)
      puts("Adelante/Derecha.");
    if(Atras & Izquierda)
      puts("Atras/Izquierda.");
    if(Atras & Derecha)
      puts("Atras/Derecha.");

    switch(toupper(getch()))
    {
      case 'Q': Adelante  = !Adelante;  Atras=0;     break;
      case 'A': Atras     = !Atras;     Adelante=0;  break;
      case 'O': Izquierda = !Izquierda; Derecha=0;   break;
      case 'P': Derecha   = !Derecha;   Izquierda=0; break;
      case ' ': Adelante=Atras=Izquierda=Derecha=0;  break;
    }
  }
}
