/*-------------------------------------------*\
 |  EJEMPLO 2.10                             |
 |  Autor: Pedro M Jimnez                  |
 |  (c) Microsystems Engineering (Bilbao)    |
\*-------------------------------------------*/

// Ejemplo para simulacin y/o MicroPIC Trainer

// La interrupcin del TMR0

// Se trata de comprobar la interrupcin provocada por el TMR0. El programa
// lee el estado de los interruptores conectados a RA0 y RA1 para reflejarlo en
// los leds conectados a RB0 y RB1 respectivamente. Al mismo tiempo el TMR0
// genera una interrupcin cada 0.25s (50ms) que se repetir 5 veces con objeto
// de hacer intermitencia de 250ms sobre el led conectado a RB7.

#include <16f84.h>

#use delay(clock=4000000)

#use fast_io(A)
#use fast_io(B)

#byte port_a = 5
#byte port_b = 6

#INT_RTCC		
timer_handler()		// Funcin de tratamiento de interrupcin para el TMR0
{
  static int cont=5;

  set_rtcc(195);	// Cargar TMR0 con 195
  cont--;
  if(!cont)
  {
    cont=5;
    if(bit_test(port_b,7))
      output_bit(PIN_B7,0);
    else
      output_bit(PIN_B7,1);
  }
    
}

main()
{
  port_b = 0;
  set_tris_a(0x1F);
  set_tris_b(0x00);

  setup_counters(RTCC_INTERNAL,RTCC_DIV_256);	// TMR0 interno con preescaler de 256
  set_rtcc(195);				// Cargar el TMR0 con 195

  enable_interrupts(RTCC_ZERO);		// Seleccionar interrupciones por rebasamiento de TMR0
  enable_interrupts(GLOBAL);		// Activar interrupciones

  while(1)
    port_b = (port_b & 0x80) | (port_a & 0x3);
}
