/*-------------------------------------------*\
 |  EJEMPLO 2.18                             |
 |  Autor: Pedro M Jimnez                  |
 |  (c) Microsystems Engineering (Bilbao)    |
\*-------------------------------------------*/

// Ejemplo para simulacin y/o MicroPIC Trainer

// El TMR0 como contador de eventos externos.

// Un sensor optoelectrnico conectado a RA4 genera un pulso cada vez que un objeto se
// interpone entre el emisor y el receptor de luz. El TMR0 se encarga de contarlos
// dependiendo del valor del preescaler. Dicho valor se ajusta mediante 3 interruptores
// (RA0-RA2) conectados a la puerta A, existiendo as 8-pre-divisiones:
//
//	A2 RA1 RA0 Divisin
//	--- --- --- --------
//	 0   0   0    1:2
//	 0   0   1    1:4
//       0   1   0    1:8
//       0   1   1    1:16
//       1   0   0    1:32
//       1   0   1    1:64
//       1   1   0    1:128
//       1   1   1    1:256

// El contaje se va visualizando en binario sobre los leds conectados a la puerta B y
// debe multiplicarse por el valor del preescaler seleccionado, para determinar el nmero
// total de pulsos. El contador se resetea al poner RA3 a "1". Debe tenerse en cuenta el
// "efecto rebote" que se produce en el interruptor RA4 de la MicroPIC Trainer.

#include <16f84.h>

#use delay(clock=4000000,RESTART_WDT)

#use fast_io(A)
#use fast_io(B)

#byte port_a = 5
#byte port_b = 6

main()
{
  port_b = 0;
  set_tris_a(0x1F);
  set_tris_b(0x00);

  setup_counters(RTCC_EXT_H_TO_L,RTCC_DIV_256);

  while(1)
  {
    restart_wdt();
    while(input(PIN_A3))
    {
      restart_wdt();
      set_rtcc(0);
    }
    
    setup_counters(RTCC_EXT_H_TO_L,(port_a & 0x07));
    port_b = get_rtcc();
  }
}
