/*-------------------------------------------*\
 |  EJEMPLO 2.23                             |
 |  Autor: Pedro M Jimnez                  |
 |  (c) Microsystems Engineering (Bilbao)    |
\*-------------------------------------------*/

// Ejemplo para simulacin y/o MicroPIC Trainer

// El manejo de la pantalla LCD.

// Este ejemplo pretende realizar un generador de mensajes para el LCD. Con RA4 a "1",
// el sistema est en el modo de programacin. En este modo el mensaje se graba en
// la EEPROM de datos. Con RA4 a "0" entramos en el modo de reproduccin. El mensaje
// grabado en la EEPROM se visualiza sobre el LCD.

// Cuando el interruptor RA3 est a nivel "1", en la posicin actual del cursor van
// apareciendo secuencialmente los distintos caracteres disponibles. Al ponerlo a "0"
// se selecciona el actual y se graba en la EEPROM. Al pasar nuevamente a "1" se
// selecciona el siguiente caracter.

// Debe recordarse que las lneas RA0-RA2 actan ahora como salida de seales de control
// hacia el LCD. Al estar conectadas con sendos interruptores en la MicroPIC Trainer,
// estos deben estar a nivel lgico "1" permanentemente.

#include <16f84.h>

#use delay(clock=4000000,RESTART_WDT)
#include <lcd2.c>

#use fast_io(A)
#use fast_io(B)


void Programacion()
{
  char c=0x20;
  int dir=0;

  lcd_enviar(LCD_COMANDO, LCD_CLEAR);
  lcd_enviar(LCD_COMANDO, LCD_HOME);
  lcd_enviar(LCD_COMANDO, LCD_DISPLAY_ON); 	// Sin cursor
  while(input(PIN_A4) && (dir < 16))
  {
    restart_wdt();
    lcd_gotoxy(1,1);
    lcd_putc(c);
    delay_ms(500);
    if(input(PIN_A3))
    {
      delay_ms(20);
      write_eeprom(dir,c);
      dir++;
      lcd_gotoxy(dir,2);
      lcd_putc(c);
      c = 0x20;
      while(input(PIN_A3))
        restart_Wdt();;
    }
    c++;
    if(c>0x80)
      c=0x20;
  }  
  write_eeprom(dir,0);
}

void Reproduccion(void)
{
  char c;
  int dir = 0;
  int i;

  lcd_enviar(LCD_COMANDO, LCD_CLEAR);
  lcd_enviar(LCD_COMANDO, LCD_HOME);
  lcd_enviar(LCD_COMANDO, LCD_DISPLAY_ON); 	// Sin cursor

  c = read_eeprom(dir);
  while(c != 0)
  {
    restart_wdt();
    lcd_putc(c);
    dir++;
    c = read_eeprom(dir);
  }
  delay_ms(2000);

  for(i=0; i<6; i++)
  {
    lcd_enviar(LCD_COMANDO, LCD_DISPLAY_ON);
    delay_ms(250);

    lcd_enviar(LCD_COMANDO, LCD_DISPLAY_OFF);
    delay_ms(250);
  }
}

main() {

  lcd_init();	// Comandos de inicializacin del LCD.
  lcd_enviar(LCD_COMANDO, LCD_CURSOR_ON);

  while(1)
  {
    restart_wdt();
    if(input(PIN_A4))
      Programacion();
    else
      Reproduccion();
  }

}

