/*-------------------------------------------*\
 |  EJEMPLO 2.24                             |
 |  Autor: Pedro M Jimnez                  |
 |  (c) Microsystems Engineering (Bilbao)    |
\*-------------------------------------------*/

// Ejemplo para simulacin y/o MicroPIC Trainer

// La memoria EEPROM de datos y el LCD. La mquina "SU TURNO", versin mejorada.

// Se trata de emular el funcionamiento de las mquinas tipo "SU TURNO" habituales
// en mltiples comercios. Sobre el LCD se visualizar el nmero del turno actual
// (2 dgitos). Este se incrementa a cada pulso aplicado por RA4. En la memoria
// EEPROM del PIC16F84 se almacena el ltimo nmero visualizado, de forma que, ante
// un fallo de alimentacin (p.e.), se reanude la cuenta en el ltimo nmero.

// Debe recordarse que las lneas RA0-RA2 actan ahora como salida de seales de control
// hacia el LCD. Al estar conectadas con sendos interruptores en la MicroPIC Trainer,
// estos deben estar a nivel lgico "1" permanentemente.

#include <16f84.h>

#use delay(clock=4000000,RESTART_WDT)

#use fast_io(A)
#use fast_io(B)

#include <lcd2.c>

int turno;

void Visualizar_Turno()
{
  lcd_gotoxy(8,1);
  lcd_putc('0'+turno/10);
  lcd_putc('0'+turno%10);
}


main() {

  lcd_init();	// Comandos de inicializacin del LCD.

  lcd_putc("Turno: ");
  turno = read_eeprom(0);
  if(turno<1 || turno>99)
    turno = 1;
  Visualizar_Turno();

  while(1)
  {
    restart_wdt();
    if(input(PIN_A4))
    {
      delay_ms(20);
      turno++;
      if(turno > 99)
        turno = 1;

      write_eeprom(0,turno);
      Visualizar_Turno();

      while(input(PIN_A4))
        restart_wdt();
      delay_ms(20);
    }
  }
}
