/*-------------------------------------------*\
 |  EJEMPLO 2.25                             |
 |  Autor: Pedro M Jimnez                  |
 |  (c) Microsystems Engineering (Bilbao)    |
\*-------------------------------------------*/

// Ejemplo para simulacin y/o MicroPIC Trainer

// El modo "sleep" y el "wake-up" (despertar) mediante el watch-dog Timer (WDT).

// Este ejemplo pretende mostrar el empleo de la funcin "sleep()" para poner al
// PIC en el modo standby de bajo consumo. El despertar del mismo se producir cada
// vez que le WDT rebase. En ese momento se producir un incremento del valor de la
// puerta B que actuar como contador binario y nuevamente se volver a la situacin
// de standby.

// El preescaler se asociar al WDT y estar comprendido entre 1 y 128, dependiendo
// del estado lgico de los interruptores RA0-RA2.

// El valor nominal del WDT es de 18ms. Es decir, con un preescaler de 1, el PIC
// "despertar" cada 18ms, con un preescaler de 128 lo har cada 2,3 segundos.


#include <16f84.h>

#use delay(clock=4000000,RESTART_WDT)
#include <lcd2.c>

#use fast_io(A)
#use fast_io(B)

#byte port_a = 5
#byte port_b = 6

main()
{
  port_b = 0;
  set_tris_a(0x1F);
  set_tris_b(0x00);

  setup_counters(RTCC_INTERNAL,WDT_18MS);

  while(1)
  {
    sleep();

    port_b++;
    setup_counters( RTCC_INTERNAL, 8 + (port_a&0x07) );  // Ver Nota.
  }  
}

// Nota:
// Esto es una copia de parte del fichero 16f84.h:
/*
		#define RTCC_DIV_2      0
		#define RTCC_DIV_4      1
		#define RTCC_DIV_8      2
		#define RTCC_DIV_16     3
		#define RTCC_DIV_32     4
		#define RTCC_DIV_64     5
		#define RTCC_DIV_128    6
		#define RTCC_DIV_256    7
		#define WDT_18MS        8
		#define WDT_36MS        9
		#define WDT_72MS       10
		#define WDT_144MS      11
		#define WDT_288MS      12
		#define WDT_576MS      13
		#define WDT_1152MS     14
		#define WDT_2304MS     15
*/
// Como se puede observar, para seleccionar un WDT de 18 ms es necesario
// pasarle a la funcin "setup_counters" el parametro WDT_18MS, es decir
// el nmero 8.
