/*-------------------------------------------*\
 |  EJEMPLO 3.11                             |
 |  Autor: Pedro M Jimnez                  |
 |  (c) Microsystems Engineering (Bilbao)    |
\*-------------------------------------------*/

// Ejemplo para MicroPIC Trainer y Trainer PLUS

// Interface I2C gestionado por software.

// Haciendo uso de las rutinas incluidas las librerias del compilador C, se trata
// de controlar el funcionamiento de los distintos dispositivos I2C, existentes
// en la tarjeta Trainer PLUS. Estos dispositivos estn conectados entre s al bus
// I2C comn. Se recomienda leer las especificaciones tcnicas de los mismos, para
// tener un mayor conocimiento de sus posibilidades. Igualmente es importane
// desconectar los leds de la MicroPIC Trainer mediante el jumper J7 pues de lo
// contrario actuaran como carga adicional en las lneas RB6 y RB7 que, es estos
// ejemplos, forman el bus I2C.

// El dispositivo I2C PCF8574. Puerto de E/S de 8 bits.

// Los leds de la Trainer PLUS que pilotan el estado de las lneas de E/S del
// PCF8574 estn complementados. Es decir, se activan cuando la lnea
// correspondiente est a "0".

// El ejemplo que se presenta a continuacin lee el estado de los interruptores
// RA0-RA4 de la MicroPIC Trainer y lo saca por la puerta de salida del
// dispositivo PCF8574.



#include <16f84.h>

#use delay(clock=4000000,RESTART_WDT)
#use I2C(MASTER,SDA=PIN_B6,SCL=PIN_B7,SLOW,RESTART_WDT)
//
// Opciones:
//  - MASTER: 	El PIC se configura como master. La otra opcin es SLAVE, y en
//		este caso hay que usar la opcin ADDRESS=n para especificar la
//		direccin del esclavo.
//
//  - SDA:	Pin del microcontrolador al que estar conectada la lnea SDA
//		del bus I2C.
//
//  - SCL:	Pin del microcontrolador al que estar conectada la lnea SDA
//		del bus I2C.
//
//  - SLOW:	Modo de baja velocidad en el bus I2C. Recomendado cuando se
//		implementa la gestin del bus I2C por software.
//		Nota: Para hacer la gestin por hardware nicamente se requiere
//		usar un PIC que lo incluya e indicar en SDA y SCL los pines a los
//		que se encuentra conectado el bus I2C.
//
//  - RESTART_WDT:
//		Cuando se estan realizando operaciones de lectura sobre el bus
//		I2C se refresca el WDT.
//
//

#byte port_a = 5

main()
{
  int dato;

  while(1)
  {
    restart_wdt();
    i2c_start();		// Condicin de inicio en el bus I2C
    i2c_write(0b01110000);	// Direccin (modo escritura) del dispositivo PCF8574
    dato = ~(port_a & 0x1F);
    i2c_write(dato);		// Se enva el dato al PCF8574
    i2c_stop();			// Condicin de stop
  }
}
