/*-------------------------------------------*\
 |  EJEMPLO 3.15                             |
 |  Autor: Pedro M Jimnez                  |
 |  (c) Microsystems Engineering (Bilbao)    |
\*-------------------------------------------*/

// Ejemplo para MicroPIC Trainer y Trainer PLUS

// Interface I2C gestionado por software.

// Haciendo uso de las rutinas incluidas las librerias del compilador C, se trata
// de controlar el funcionamiento de los distintos dispositivos I2C, existentes
// en la tarjeta Trainer PLUS. Estos dispositivos estn conectados entre s al bus
// I2C comn. Se recomienda leer las especificaciones tcnicas de los mismos, para
// tener un mayor conocimiento de sus posibilidades. Igualmente es importane
// desconectar los leds de la MicroPIC Trainer mediante el jumper J7 pues de lo
// contrario actuaran como carga adicional en las lneas RB6 y RB7 que, es estos
// ejemplos, forman el bus I2C.

// El dispositivo I2C SAA1064. Controlador de displays 7 segmentos.

// Se trata de un contador UP/DOWN de 4 dgitos cuya visualizacin se realiza
// en los 4 displays de la MicroPIC Trainer PLUS, gobernados por el dispositivo
// SAA1064

// Los pulsos a contar se introducen por RA0. Si RA1 == 0 la cuenta es accendente,
// en caso contrario descendente.
#include <16f84.h>

#use delay(clock=4000000,RESTART_WDT)
#use I2C(MASTER,SDA=PIN_B6,SCL=PIN_B7,SLOW,RESTART_WDT)

#use fast_io(A)

unsigned int const disp[10]={0x3F,0x06,0x5B,0x4F,
                             0x66,0x6D,0x7D,0x27,
                             0x7F,0x6F};

long int d_mil=0,d_cen=0,d_dec=0,d_uni=0;
signed long int n=0;

void Visualizar(void)
{
  long int temp;

  d_mil = n / 1000;
  temp = n - d_mil*1000;
  d_cen = temp / 100;
  d_dec = (temp - d_cen*100) / 10;
  d_uni = n % 10;

  i2c_start();
  i2c_write(0b01110110); 	// Direccin (Escritura)
  i2c_write(0b00000000); 	// Instruccin
  i2c_write(0b01000111); 	// Control
  i2c_write(disp[d_mil]); 	// Unidades de millar
  i2c_write(disp[d_cen]); 	// Centenas
  i2c_write(disp[d_dec]); 	// Decenas
  i2c_write(disp[d_uni]); 	// Unidades
  i2c_stop();
}


main()
{
  Visualizar();
  while(1)
  {
    restart_wdt();

    if(input(PIN_A0))
    {
      delay_ms(20);

      while(input(PIN_A0))
        restart_wdt();

      if(input(PIN_A1))
        n++;
      else
        n--;

      if(n>9999)
        n=0;
      if(n<0)
        n=9999;

      Visualizar();
    }
  }
}
