;
;					EJEMPLO 3.19
;
;				Autor: Mikel Etxebarria
;			(c) Microsystems Engineering (Bilbao)
;
;Ejemplo para MicroPIC Trainer y Trainer PLUS. Interface I2C gestionado por software.
;
;Haciendo uso de las rutinas incluidas en el fichero I2CLOW.INC, se trata de controlar
;el funcionamiento de los distintos dispositivos I2C, existentes en la tarjeta 
;Trainer PLUS. Estos dispositivos estn conectados entre s al bus I2C comn. Se recomienda
;leer las especificaciones tcnicas de los mismos, para tener un mayor conocimiento de las
;posibilidades de los mismos. Igualmente es importante desconectar los leds de la uPIC Trainer
;mediante el jumper J7 pues de lo contrario actuaran como carga adicional en las lneas RB6
;y RB7 que, en estos ejemplos, forman el bus I2C.
;
;El dispositivo I2C PCF8591. Convertidor DAC y ADC
;
;El programa explora el teclado de uPIC Trainer Plus y saca por la puerta de salida
;del PCF8574 el valor binario del octeto tecleado. Este valor se ver reflejado en
;los leds de uPIC Trainer Plus que pilotan los niveles lgicos presentes en las lneas
;de E/S. Posteriormente dicho octeto se enva al DAC del PCF8591 que generar una 
;tensin analgica proporcional. Dicha tensin est disponible en la borna Aout de
;uPIC Trainer Plus y podr ser medida con un polmetro

		List	p=16F84		;Tipo de procesador
		include	"P16F84.INC"	;Definiciones de registros internos

CLKIN		equ	.4000000	;Frecuencia de trabajo
                 
I2c_var		equ	0x0c		;Inicio de las variables para rutinas I2C_LOW
Key_var		equ	0x12		;Inicio de las variables de manejo del teclado		
A_out		equ	0x18		;Variable analgica de salida

		org	0x00		;Vector de reset
		goto	Inicio
		org	0x05		;Salvar el vector de interrupcin	

		include	"I2CLOW.INC"	;Incluye las rutinas de manejo del bus I2C
		include "teclado.INC"	;Incluye rutinas de manejo del teclado

;**************************************************************************************
;A_Output: Esta rutina recoge el valor presente en la variable A_out y lo saca por el DAC
;del dispositivo PCF8591

A_Output:	call	Txt_Start_Bit	;Enva condicin de inicio
		movlw	b'10010000'
		movwf	DataByte
		call	Txt_Byte	;Transmite direccin del PCF8591 (escritura)
		movlw	b'01000000'
		movwf	DataByte
		call	Txt_Byte	;Transmite byte de control del PCF 8591
		movf	A_out,W
		movwf	DataByte
		call	Txt_Byte	;Transmite valor a convertir en analgico
		call	Txt_Stop_Bit	;Transmite secuencia de STOP
		return

;**************************************************************************************
;SALIDA: Esta rutina recoge el valor del octeto a sacar y va I2C lo transmite al PCF
;8574 para que lo represente en los leds conectados a sus lneas de E/S.

SALIDA:		call	Inicia_Bus	;Inicia lneas del bus I2C
		call	Txt_Start_Bit	;Enva condicin de inicio
		movlw	b'01110000'
		movwf	DataByte
		call	Txt_Byte	;Transmite direccin del PCF8574 (escritura)
		comf	A_out,W		;Complementa el octeto a sacar puesto que
					;los leds pilotos conectados a las lneas
					;son activos por nivel "0"
		movwf	DataByte
		call	Txt_Byte	;Transmite el octeto a sacar.		
		call	Txt_Stop_Bit	;Transmite secuencia de STOP
		return

Inicio:		clrf	PORTA
		clrf	PORTB
		bsf	STATUS,RP0	;Selecciona pgina 1 de datos
		movlw	b'00011111'
		movwf	TRISA		;RA0-RA4 entradas
		clrf	TRISB		;Puerta A y B salidas
		movlw	b'10000110'	;Preescaler de 128 para el TMR0. Pull_Up Off
		movwf	OPTION_REG				
		bcf	STATUS,RP0	;Selecciona pgina 0 de datos	

		call	Inicia_Bus	;Inicia el bus I2C
				
Loop:		clrwdt			;Refresca el WDT
		clrf	A_out
		call	Key_Scan	;Explora el teclado
		movlw	0x80
		subwf	Tecla,W
		btfsc	STATUS,Z	;Hay alguna pulsada ??
		goto	Loop		;No
		call	Key_BCD		;Si.Convierte cdigo de tecla a BCD
		swapf	Tecla,w
		movwf	A_out		;Nible de ms peso del octeto a sacar				

Loop_0_1	clrwdt
		call	Key_Scan	;Explora el teclado
		movlw	0x80
		subwf	Tecla,W
		btfss	STATUS,Z	;Hay alguna pulsada ??
		goto	Loop_0_1	;Si, esperar

Loop_1:		clrwdt			;No, ya se leber la tecla. Refrescar WDT
		call	Key_Scan	;Explora el teclado
		movlw	0x80
		subwf	Tecla,W
		btfsc	STATUS,Z	;Hay alguna pulsada ??
		goto	Loop_1		;No
		call    Key_BCD		;Si. Convierte cdigo de tecla a BCD
		movf	A_out,W
		iorwf	Tecla,W		;Concatena el 2 nible del octeto a sacar				
		movwf	A_out														
		call	SALIDA		;Salida del octeto a los leds del PCF8574
		call	A_Output	;Salida del octeto a travs del DAC del PCF8591

Loop_1_1	clrwdt
		call	Key_Scan	;Explora el teclado
		movlw	0x80
		subwf	Tecla,W
		btfss	STATUS,Z	;Hay alguna pulsada ??
		goto	Loop_1_1	;Si, esperar		

		goto	Loop		;No, la tecla ha sido liberada
			
		end			;Fin del programa fuente

