;
;					EJEMPLO 3.7
;
;				Autor: Mikel Etxebarria
;			(c) Microsystems Engineering (Bilbao)
;
;Ejemplo para MicroPIC Trainer y Trainer PLUS. Comunicacin serie gestionada por software.
;
;Haciendo uso de las rutinas incluidas en el fichero RS232LOW.INC, se trata de utilizar el
;canal RS232 de la Trainer PLUS para comunicar va serie con diferentes perifricos. En los
;ejemplos aqu propuestos se utilizar un PC para que, con el software TERMINAL.EXE, actue
;como un simple terminal de comunicaciones. El usuario deber preparar los cables de comuni-
;cacin entre la Trainer PLUS y el PC, teniendo en cuenta de cruzar la seal de Transmisin
;(TxD) con la de recepcin (RxD) y viceversa.
;
;Este ejemplo es similar al anterior. Propone esperar a recibir un carcter desde el ter-
;minal para, inmediatamente, retransmitirlo al mismo a modo de ECO y visualizarlo sobre la
;pantalla LCD de la uPIC Trainer.

		List	p=16F84		;Tipo de procesador
		include	"P16F84.INC"	;Definiciones de registros internos

CLKIN		equ	.4000000	;Frecuencia de trabajo
BAUDIOS		equ	.9600		;Velocidad de comunicacin
T_MODO		equ	1		;Transmite primero bit LSB
R_MODO		equ	1		;Recibir primero bit LSB
T_Nbit		equ	8		;Transmite caracteres de 8 bits
R_Nbit		equ	8		;Recibe caracteres de 8 bits
Sbit		equ	2		;Transmite 2 bits de stop
                 
Lcd_var		equ	0x0c		;Inicio de las variables para rutinas LCD
Rs232_var	equ	0x0e		;Inicio de las variables para rutinas RS232
Lineas		equ	0x13		;Contador de lneas
Caracteres	equ	0x14		;Contador de caracteres
					
		org	0x00		;Vector de reset
		goto	Inicio
		org	0x05		;Salvar el vector de interrupcin	
					
		include	"RS232LOW.INC"	;Incluir rutinas de comunicacin
		include "LCD_CXX.INC"	;Incluir rutinas LCD							

Inicio:		bsf	STATUS,RP0	;Selecciona pgina 1 de datos
		movlw	b'10001111'	;Preescaler de 128 para el WDT. Pull_Up Off
		movwf	OPTION_REG	
		movlw	b'11101111'
		movwf	TRISB		;RB5 entrada y RB4 salida
		bcf	STATUS,RP0	;Selecciona pgina 0 de datos	

		call	UP_LCD		;Inicia E/S para el mdulo LCD
		call	LCD_INI		;Inicializacin del mdulo LCD
		movlw	b'00001111'
		call	LCD_REG		;LCD on, Cursor On y Blink On

LCD_HOME	movlw	b'00000001'
		call	LCD_REG		;Borra LCD y Home
		movlw	.16
		movwf	Caracteres	;Inicia contador de caracteres
		movlw	.2
		movwf	Lineas		;Inicia contador de lneas
				
Loop		clrwdt
		bsf	STATUS,RP0	;Selecciona pgina 1
		movlw	b'11101111'
		movwf	TRISB		;RB5 entrada y RB4 salida
		bcf	STATUS,RP0

		bsf	Txd_pin		;Lnea de transmisin a "1"
		bsf	Rxd_pin		;Lnea de recepcin a "1"

		call	RxD		;Espera a recibir un carcter
		movf	Rxdreg,W	;Lee el carcter recibido
		movwf	Txdreg
		call	TxD		;Retransmite carcter recibido (ECO)
		call	UP_LCD		;Reconfigura E/S para el LCD
		movlw	.13
		subwf	Rxdreg,W	;Mira si carcter recibido es CR
		btfsc	STATUS,Z	;No
		goto	LCD_HOME	;Si, borra LCD
		movf	Rxdreg,W
		call	LCD_DATO	;Visualiza carcter en el LCD

		decfsz	Caracteres,F	;Decrementa contador de caracteres
		goto	Loop

		movlw	.16
		movwf	Caracteres	;Inicia contador de caracteres
		movlw	0xc0
		call	LCD_REG		;Selecciona segunda lnea del LCD
		decfsz	Lineas,F	;Se han completado 2 lneas ??
		goto	Loop		;No		
		goto	LCD_HOME	;Si, borrar pantalla

		end			;Fin del programa fuente

