;
;					EJEMPLO 17
;
;				Autor: Mikel Etxebarria
;			(c) Microsystems Engineering (Bilbao)
;
;Ejemplo MicroPIC Trainer y Micro'PIC I/O
;
;Los mdulos CCPx. Modo de Comparacin. Disparo de eventos
;
;Los mdulos CCPx pueden disparar, en el modo de comparacin, diferentes eventos al coinci-
;dir el valor del TMR1 con el valor prefijado en los registros CCPRxL y CCPRxH. El mdulo 
;CCP1 concretamente, puede provocar un reset del TMR1 ponindolo a 0. De esta forma, los 
;registros CCPR1L y CCPR1H, actan como registros de periodo.
;		
;El ejemplo propone el control del N de vueltas o pasos que gira el motor DC alojado en la
;tarjeta Micro'PIC I/O. Un encoder proporciona un pulso por cada paso dado. Este se aplica,
;mediante RC0/T1CKI, al TMR1 que los va contando. Cuando se alcanza el valor establecido en
;la constante "Horario", el motor se detiene, se realiza una temporizacin y se inicia el
;movimiento en sentido contrario tantos pasos como indique la constante antihorario.
;
;Las seales de salida RB0 y RB1 gobiernan el sentido de giro y RB2 el On/Off del motor.

		List	p=16F873	;Tipo de procesador
		include	"P16F873.INC"	;Definiciones de registros internos

Horario		equ	.500		;Pasos de giro horario
Antihorario	equ	.300		;Pasos de giro antihorario
Delay_Cont	equ	0x20		;Variable de temporizacin
Giro		equ	0x21		;Flag sealizador de sentido de giro

		org	0x00		;Vector de Reset
		goto	Inicio

		org	0x04		
	
;Programa de tratamiento de la interrupcin 

Inter		bcf	PIR1,CCP1IF	;Reponer el flag del mdulo CCP1
		clrf	PORTB		;Motor en Off

;Se temporiza 50 ms tantas veces como indique la variable delay. El TMR0 se carga con 195
;que, con un preescaler de 256, rebasa al de 50mS.

Delay		movlw	~.195
		movwf	TMR0
Delay_50	clrwdt			;Refrescar el WDT
		btfss	INTCON,T0IF	;Han pasado 50 mS ??
		goto	Delay_50	;No, esperar
		bcf	INTCON,T0IF	;Reponer el flag del TMR0
		decfsz	Delay_Cont,F	;Se ha repetido tantas veces indique Delay_Cont ??
		goto	Delay		;Todava no, temporizar otros 50 mS
		movlw	.20
		movwf	Delay_Cont	;Iniciar variable de temporizacin
		btfsc	Giro,0		;Giro horario ??
		goto	G_Horario	;No, giro horario

G_Anti_horario	incf	Giro,F		;Selecciona prximo giro horario
		movlw	high Antihorario
		movwf	CCPR1H
		movlw	low  Antihorario
		movwf	CCPR1L		;Ajusta N de pasos anti-horario
		movlw	b'00000101'
		movwf	PORTB		;Giro anti-horario
		retfie

G_Horario	incf	Giro,F		;Selecciona prximo giro antihorario
		movlw	high Horario
		movwf	CCPR1H
		movlw	low Horario
		movwf	CCPR1L		;Ajusta N de pasos horario
		movlw	b'00000110'
		movwf	PORTB		;Giro horario
		retfie

;Programa principal

Inicio	        clrf	PORTB		;Desconecta salidas
		clrf	PORTC
		bsf	STATUS,RP0	;Selecciona banco 1
		movlw	b'00000110'
		movwf	ADCON1		;Puerta A E/S digitales
		clrf	TRISB		;Puerta B se configura como salida
		movlw	b'11111111'
		movwf	TRISC		;RC0 entrada
		movlw	b'11000111'
		movwf	OPTION_REG	;Preescaler de 256 asociado al TMR0
		bsf	PIE1,CCP1IE	;Habilita interrupcin del mdulo CCP1
		bcf	STATUS,RP0	;Selecciona banco 0
	
;El mdulo CCP1 acta en modo de comparacin con disparo de eventos. Los registros CCPR1L y
;CCPR1H forman un registro de 16 bits que acta a modo de registro de periodos, provocando 
;que el TMR1 se reinicie cada vez que alcance el valor prefijado en esos registros.

		movlw	b'00001011'
		movwf	CCP1CON		;CCP1 en modo comparacin con disparo de eventos

		movlw	high Horario
		movwf	CCPR1H
		movlw	low Horario
		movwf	CCPR1L		;Carga el N de pasos en sentido horario
		clrf	TMR1L
		clrf	TMR1H		;Puesta a 0 del TMR1
		
;El TMR1 acta en el modo contador de pulsos externos (los que genera el encoder) y con un 
;preescaler de 1:1. 

		movlw	b'00000011'
		movwf	T1CON		;TMR1 en On

		movlw	.20
		movwf	Delay_Cont	;Inicia variable de temporizacin (1 seg.)
		clrf	Giro		;Inicia flag de giro horario

		movlw	b'11000000'
		movwf	INTCON		;Habilita interrupciones
		movlw	b'00000110'
		movwf	PORTB		;Motor On, giro horario

Loop		clrwdt			;Refresca el WDT	
		goto	Loop		;Bucle infinito		

		end			;Fin del programa fuente
