;
;					EJEMPLO 25
;
;				Autor: Mikel Etxebarria
;			(c) Microsystems Engineering (Bilbao)
;
;Ejemplo MicroPIC Trainer y Micro'PIC I/O
;
;El mdulo conversor ADC. Medida de temperatura ambiente.
;
;Los dispositivos PIC16F87X disponen de un convertidor A/D de 10 bits de resolucin y 5 u 8 
;canales de entrada analgica. La tensin de referencia determina la resolucin por bit:
;(Res. = Vref/1024). Con 5 Vref=4.86 mV/Bit; con 2.5 Vref=2.43 mV/Bit
;
;El sensor de t de la tarjeta Micro'PIC I/O proporciona una tensin analgica de 10mV/C,
;que se aplica al canal ADC RA5/AN4. El ADC trabaja con una tensin de referencia externa de
;2.5V que se obtiene desde la tarjeta Micro'PIC I/O y se aplica en RA3/Vref. Aproximadamente
;se toma una muestra de la temperatura actal cada segundo
;
;La pantalla LCD visualiza la temperatura.

		List	p=16F873	;Tipo de procesador
		include	"P16F873.INC"	;Definiciones de registros internos

Lcd_var		equ	0x20		;Variables para las rutinas del LCD
DatoA_H		equ	0x22
DatoA_L		equ	0x23		;Multiplicando de 16 bits
DatoB_H		equ	0x24
DatoB_L		equ	0x25		;Multiplicador de 16 bits
DatoD_H		equ	0x26
DatoD_L		equ	0x27
DatoC_H		equ	0x28
DatoC_L		equ	0x29		;Resultado de 32 bits
Contador	equ	0x2a		;Variable contadora
BCD_2		equ	0x2b
BCD_1		equ	0x2c
BCD_0		equ	0x2d		;Variables BCD
Temporal	equ	0x2e		;Variable temporal
Delay		equ	0x2f		;Variable de temporizacin

		org	0x00		;Vector de Reset
		goto	Inicio

		org	0x05		;Salva el vector de interrupcin
		
		include "LCD_CXX.INC"	;Incluye rutinas de manejo del LCD

;Mul16x16: Esta rutina multiplica sin signo dos nmeros de 16 bits (DatoB_H:DatoB_L * 
;DatoA_H:DatoA_L) el resultado de 32 bits se deposita en (DatoD_H:DatoD_L y DatoC_H:DatoC_L)

Mul16x16	movlw	.16
		movwf	Contador	;Inicia contador
		clrf	DatoD_H
		clrf	DatoD_L		;Borra parte alta del resultado

Mul_Loop	rrf	DatoB_H,F
		rrf	DatoB_L,F
		btfss	STATUS,C	;El bit LSB es "1" ?
		goto	No_suma		;No
		movf	DatoA_L,W
		addwf	DatoD_L,F	;Suma octeto de menos peso
		btfsc	STATUS,C
		incf	DatoD_H,F
		movf	DatoA_H,W
		addwf	DatoD_H,F	;Suma octeto de ms peso
No_suma		rrf	DatoD_H,F
		rrf	DatoD_L,F
		rrf	DatoC_H,F
		rrf	DatoC_L,F
		decfsz	Contador,F
		goto	Mul_Loop
		return

;16Bits_BCD: Esta rutina convierte un nmero binario de 16 bits situado en DatoC_H y
;DatoC_L en 5 dgitos BCD que se depositan en las variables BCD_0, BCD_1 y BCD_2, siendo 
;esta ltima la de menos peso. Est presentada en la nota de aplicacin AN544 de MICROCHIP 
;y adaptada por Microsystems Engineering

Bits16_BCD	bcf	STATUS,C
		clrf	Contador	
		bsf	Contador,4	;Carga el contador con 16		
		clrf	BCD_0
		clrf	BCD_1
		clrf	BCD_2		;Puesta a 0 inicial

Loop_16		rlf	DatoC_L,F
		rlf	DatoC_H,F
		rlf	BCD_2,F
		rlf	BCD_1,F
		rlf	BCD_0,F		;Desplaza a izda. (multiplica por 2)
		decfsz	Contador,F
		goto	Ajuste
		return

Ajuste		movlw	BCD_2
		movwf	FSR		;Inicia el ndice
		call	Ajuste_BCD	;Ajusta el primer byte
		incf	FSR,F
		call	Ajuste_BCD	;Ajusta el segundo byte
		incf	FSR,F
		call	Ajuste_BCD
		goto	Loop_16

Ajuste_BCD	movf	INDF,W		
		addlw	0x03
		movwf	Temporal	
		btfsc	Temporal,3	;Mayor de 7 el nibble de menos peso ??
		movwf	INDF		;Si, lo acumula
		movf	INDF,W		
		addlw	0x30
		movwf	Temporal
		btfsc	Temporal,7	;Mayor de 7 el nibble de menos peso ??
		movwf	INDF		;Si, lo acumula
		return

;Programa principal

Inicio	        clrf	PORTA
		clrf	PORTB
		bsf	STATUS,RP0	;Selecciona banco 1
		movlw	b'00000110'
		movwf	ADCON1		;Puerta A digital
		clrf	TRISB		;Puerta B se configura como salida
		clrf	TRISA		;RA4-RA0 salidas
		movlw	b'11000111'
		movwf	OPTION_REG	;Preescaler de 128 asociado al TMR0
		bcf	STATUS,RP0	;Selecciona banco 0

		call	UP_LCD		;Configuracin de E/S para el LCD
		call	LCD_INI		;Inicializacin del LCD
		movlw	b'00001100'
		call	LCD_REG		;LCD On, cursor y Blink Off

Loop		bsf	STATUS,RP0	;Selecciona pgina 1
		movlw	b'00101000'
		movwf	TRISA		;RA3 y RA5 entradas
		movlw	b'10000001'	;Justificacin dcha. RA5/AN4 entrada analgica
		movwf	ADCON1		;RA3/Vref entrada de tensin de referencia
		bcf	STATUS,RP0	;Selecciona pgina 0

;ADC en On con frecuencia de Fosc/32 y selecin del canal RA5/AN4

		movlw	b'10100001'
		movwf	ADCON0

;Temporizar 1 segundo mediante el TMR0. Este, con un preescaler de 256, evoluciona cada
;256uS. Se carga con 195 para realizar una temporizacin de 50mS. Los 50 mS se repiten 20 
;veces para conseguir un segundo. Seguidamente se toma una muestra de la temperatura.

		movlw	.20
		movwf	Delay
Delay_1		movlw	~.195
		movwf	TMR0		;Inicia el TMR0
		bcf	INTCON,T0IF	;Restaura el flag del TMR0
Delay_2		clrwdt
		btfss	INTCON,T0IF	;Han pasado 50 mS ??
		goto	Delay_2		;Todava no
		decfsz	Delay,F		;Se ha repetido 20 veces ??
		goto	Delay_1		;No

		bcf	PIR1,ADIF	;Restaurar el flag del ADC
		bsf	ADCON0,GO	;Inicio de conversin
ADC_Wait	btfss	PIR1,ADIF	;Fin de conversin ??
		goto	ADC_Wait	;Todava no, esperar

;Al resultado de la conversin se le multiplica por 243 (2.43) que es la resolucin por bit.
;P.Ej. La T es de 25.6, como el sensor proporciona 10mV/C la tensin a convertir ser de
;256 mV. Siendo la resolucin de 2.43mV/bit el resultado de la conversin dar 0x69 (105)
;que, multiplicado por 0xf3 (243) queda un resultado de 0x63ab (25515). Se desplaza la coma 
;tres posiciones a la izda. y queda finalmente 25.5 (un error de +/- 1 bit)

		bsf	STATUS,RP0	;Banco 1
		movf	ADRESL,W
		bcf	STATUS,RP0
		movwf	DatoB_L
		movf	ADRESH,W
		movwf	DatoB_H		;Carga el multiplicando (valor de conversin)
		movlw	.243
		movwf	DatoA_L
		clrf	DatoA_H		;Carga el multiplicador (243)

		call	Mul16x16	;Multiplica
		call	Bits16_BCD	;Convierte a BCD
		
;Visualiza el resultado sobre la pantalla LCD. La pantalla LCD se actualiza por tanto a cada
;segundo
		
		bsf	STATUS,RP0	;Selecciona banco 1
		movlw	b'00000110'
		movwf	ADCON1		;Puerta A digital
		bcf	STATUS,RP0	;Selecciona banco 0
		call	UP_LCD		;Reconfigura puertos para el LCD
		movlw	0x80
		call	LCD_REG		;Posiciona el cursor
		movf	BCD_0,W
		andlw	0x0f
		iorlw	0x30
		call	LCD_DATO	;Visualiza decenas de grado
		swapf	BCD_1,W
		andlw	0x0f
		iorlw	0x30
		call	LCD_DATO	;Visualiza unidades de grado
		movlw	'.'
		call	LCD_DATO	;Visualiza punto decimal
		movf	BCD_1,W
		andlw	0x0f
		iorlw	0x30
		call	LCD_DATO	;Visualiza dcimas de grado
		movlw	' '
		call	LCD_DATO	;Visualiza un ' '
		movlw	0xdf
		call	LCD_DATO	;Visualiza smbolo 
		movlw	'C'
		call	LCD_DATO	;Visualiza C
		goto	Loop

		end			;Fin del programa fuente
