;
;					EJEMPLO 30
;
;				Autor: Mikel Etxebarria
;			(c) Microsystems Engineering (Bilbao)
;
;Ejemplo MicroPIC Trainer y Micro'PIC I/O
;
;El mdulo conversor ADC. Regulacin en alterna. Variacin controlada de potencia. 
;
;Los dispositivos PIC16F87X disponen de un convertidor A/D de 10 bits de resolucin y 5 u 8 
;canales de entrada analgica. La tensin de referencia determina la resolucin por bit:
;(Res. = Vref/1024). Con 5 Vref=4.86 mV/Bit; con 2.5 Vref=2.43 mV/Bit
;
;El ngulo de disparo del opto-triac de la tarjeta Micro'PIC I/O puede variarse y con ello
;la potencia aplicada a la carga (lmpara). Una vez sincronizada la alterna en su paso por
;0, el disparo del TRIAC se puede retrasar entre 0 (mxima potencia) y 10 mS (mnima 
;potencia).
;
;En el ejemplo, una tensin analgica aplicada por RA3, se convierte en un valor digital. 
;Dicho valor es el que determina el delay de retraso en el disparo del Opto-Triac y, regula 
;por tanto la potencia aplicada a la carga.
;
;La seal RB0/INT detecta el cruce por cero de la tensin alterna. RB1 se conecta al Opto-
;Triac para provocar el disparo o bloqueo del mismo. Por RA3/AN3 se aplica la tensin anal-
;gica a convertir. Esta procede del potencimetro AN3 de la Trainer o de la clula LDR de la
;Micro'PIC Trainer I/O.

		List	p=16F873	;Tipo de procesador
		include	"P16F873.INC"	;Definiciones de registros internos

Delay_L		equ	0x20
Delay_H		equ	0x21		;Variables de temporizacin

		org	0x00		;Vector de Reset
		goto	Inicio

		org	0x04		;Vector de interrupcin

;Tratamiento de interrupcin. 

Inter		bcf	PORTB,1		;Triac en Off
		bcf	INTCON,INTF	;Restaura flag de interrupcin RB0/INT
		bsf	ADCON0,GO	;Inicia la conversin del canal RA3/AN3
		bcf	PIR1,ADIF	;Restaura flag del ADC
ADC_Wait	btfss	PIR1,ADIF	;Fin de conversin ??
		goto	ADC_Wait	;Todava no

		bsf	STATUS,RP0	;Selecciona el banco 1
		movf	ADRESL,W
		bcf	STATUS,RP0
		movwf	Delay_L		;Salva los 8 bits de menos meso de la conversin
		movf	ADRESH,W
		movwf	Delay_H		;Salva los bits de ms peso de la conversin
	
;El ADC de 10 bits genera un resultado mnimo de 0 y mximo de 1023. Si se multiplica por
;8 se obtiene un valor comprendido entre 0 y 8184. Si dicho valor expresa microsegundos,
;se obtiene un delay entre 0 y 8 mS para el retraso en el disparo del Triac. La carga no
;llegar nunca a desconectarse del todo. Esto slo ocurrira con un retraso de 10 mS

		bcf	STATUS,C
		rlf	Delay_L,F
		rlf	Delay_H,F
		rlf	Delay_L,F
		rlf	Delay_H,F
		rlf	Delay_L,F
		rlf	Delay_H,F	;Multiplica por 8

		comf	Delay_L,W
		movwf	TMR1L
		comf	Delay_H,W
		movwf	TMR1H		;Carga el TMR1
		bcf	PIR1,TMR1IF	;Restaura el flag del TMR1
TMR1_Wait	btfss	PIR1,TMR1IF	;Ha finalizado la temporizacin para el disparo ??
		goto	TMR1_Wait	;Todava no
		bsf	PORTB,1		;Ahora si, Triac On
		retfie

;Programa principal

Inicio	        clrf	PORTB		
		bsf	STATUS,RP0	;Selecciona banco 1
		movlw	b'11111101'
		movwf	TRISB		;RB0 entrada. RB1 salida
		movlw	b'10000000'
		movwf	ADCON1		;Puerta A analgica. Vref.=Vdd
		movlw 	b'00001111'
		movwf	TRISA		;RA3 entrada analgica
		movlw	b'11001111'
		movwf	OPTION_REG	;Preescaler de 128 asociado al WDT
		bcf	STATUS,RP0	;Selecciona banco 0

;Se seleciona el canal RA3/AN3. ADC con frecuencia Fosc./8

		movlw	b'01011001'
		movwf	ADCON0		;Mdulo ADC en On

;Con un preescaler de 1:1, el TMR1 evoluciona cada 1 uS al trabajar a una frecuencia de 4MHz

		movlw	b'00000001'
		movwf	T1CON		;TMR1 en On

		movlw	b'10010000'
		movwf	INTCON		;Habilita interrupcin externa por RB0/INT

Loop		clrwdt
		goto	Loop

		end			;Fin del programa fuente
